
##
## technical details ..
##
TailOfRelator := function( C, g, i, j )
    local e, a, b;
    e := RelativeOrdersOfPcp( C.factor )[i];
    if i = j  then
        a := g[i]^e;
    elif j < i  then
        a := g[i]^g[j];
    else
        a := g[i]^(g[j-i]^-1);
    fi;
    if Length(C.relators[i][j]) > 0 then
        b := Product(List( C.relators[i][j], x -> g[x[1]]^x[2] ));
    else
        b := g[1]^0;
    fi;
    return ExponentsByPcp(C.t, b^-1 * a);
end;


##
## the two-cohomology of G
##
TwoCohomInfoCG := function( G )
    local C, cc, cn, cb, cf, c, l;

    # get info
    AddExtensionInfoCG(G);
    C := G!.crrec;

    # get cocyles and coboundaries with nice bases
    cc := TwoCocyclesCR(C);
    cc := LLLReducedBasis(cc).basis;
    TriangulizeIntegerMat(cc);
    cb := TwoCoboundariesCR(C);
    TriangulizeIntegerMat(cb);

    # compute quotient
    cf := AdditiveFactorPcp(cc, cb, 0);
    cf.full := Concatenation(cf.prei, cf.denom);
    G!.cohom := cf;

    # get cocycle associated with G
    c := List( C.enumrels, x -> TailOfRelator( C, C.f, x[1], x[2] ) );
    G!.extcc := Concatenation(c);

    # find expression of c with respect to basis of cc
    if Length(cf.full) = 0 then return []; fi;
    l := SolutionIntMat(cf.full, G!.extcc){[1..Length(cf.prei)]};
    return List([1..Length(l)], x -> l[x] mod cf.rels[x]);
end;


##
## technical details ...
##
TailImage := function(G, c, tup)
    local C, n, F, f, H, h, a, k, t;

    # get info
    AddExtensionInfoCG(G);
    C := G!.crrec;
    n := Length(C.f);
    F := C.factor!.group;
    f := Igs(F);

    # construct extension
    H := ExtensionCR( C, c );
    h := Pcp(H);

    # map with tup[2]^-1
    a := Inverse(tup[2]);
    k := List([1..n], x -> MappedVector(Exponents(Image(a, f[x])),h{[1..n]}));

    # push through relators
    t := List( C.enumrels, x -> TailOfRelator( C, k, x[1], x[2] ) );
    t := List( t, x -> x * tup[1] );
    return Concatenation(t);
end;


##
## determine the action of a compatible pair on the two-cohomology
##
MatrixOperOfTupleCG := function(G, tup)
    local cf, bs, ig, c, l;
    cf := G!.cohom;
    ig := [];
    for c in cf.prei do
        l := TailImage(G, c, tup);
        l := SolutionIntMat(cf.full, l){[1..Length(cf.prei)]};
        l := List([1..Length(l)], x -> l[x] mod cf.rels[x]);
        Add(ig, l);
    od;
    return ig;
end;

