
CHECK := true;

FormSpace := function(P)
    local d, g, B, h, k, M, i, j, n; 

    d := Length(One(P));
    if Size(P) = 1 then 
        g := [One(P)];
    else
        g := SmallGeneratingSet(P);
    fi;
    B := Basis(MatrixAlgebra(Rationals,d));
    h := List(g, x -> List(B, y -> Coefficients(B, TransposedMat(x)*y*x)));
    k := List(h, x -> x - x^0);
    M := List([1..d^2], y -> Concatenation(List(k, x -> x[y])));
    n := NullspaceMat(M);
    n := List(n, x -> x*B);

    if CHECK then 
        for i in [1..Length(n)] do
            for j in [1..Length(g)] do
                if TransposedMat(g[j])*n[i]*g[j] <> n[i] then 
                    Error("wrong form");
                fi;
            od;
        od;
    fi;

    return n;
end;

GetEqns := function(P)
    local d, U, u, f, b, x, M, i, j, k, l, L, g;

    # construct basis
    d := Length(One(P));
    U := FormSpace(P);
    u := List(U, x -> Flat(x));
    f := BaseSteinitzVectors(IdentityMat(d^2), u).factorspace;
    b := Concatenation(f,u);
   
    # construct M
    x := List([1..d^2], x -> Indeterminate(Rationals, x));
    M := MutableNullMat(d^2, d^2);
    for i in [1..d] do
        for j in [1..d] do
            for k in [1..d] do
                for l in [1..d] do
                    M[d*(i-1)+j][d*(k-1)+l] := x[d*(i-1)+l]*x[d*(j-1)+k];
                od;
            od;
        od;
    od;
    
    L := b*M*b^-1;
    g := Flat(L{[Length(f)+1..d^2]}{[1..Length(f)]});
    if IsInt(d/2) then 
        Add(g, Determinant(CutVector(x,d))^2-1);
    else
        Add(g, Determinant(CutVector(x,d))-1);
    fi;
    g := Filtered(g, x -> x <> 0*x);
    return Set(g);
end;

GetInfo := function(n)
    local g, f, k, e, i, u, l;

    if n = 2 then 
        g := Set(List([1..17],x -> PointGroup(SpaceGroupIT(2,x))));
    elif n = 3 then 
        g := Set(List([1..230],x -> PointGroup(SpaceGroupIT(3,x))));
    elif n = 4 then
        g := Set(List([1..4783], x -> PointGroupByNumber(4,x)));
    fi;

    f := List(g, FormSpace);
    k := []; 
    e := Set(f);
    for i in [1..Length(e)] do
        u := g{Filtered([1..Length(g)], x -> f[x]=e[i])};
        l := List(u, Size);
        SortParallel(l,u);
        Add(k, u[Length(u)]);
    od;

    return rec( grps := k, forms := f);
end;
        
